package pomodoro

import (
	"sync"
	"time"

	"git.smsvc.net/pomodoro/GoTomato/internal/shared"
	"git.smsvc.net/pomodoro/GoTomato/pkg/models"
)

var mu sync.Mutex // to synchronize access to shared state
var timer Timer

func waitForTimer(t Timer) bool {
	for {
		select {
		case <-t.End:
			return true
		case <-t.Abort:
			return false
		case shared.State.TimeLeft = <-t.TimeLeft:
			// do nothing, just let the timer update the message
		}
	}
}

// RunPomodoro iterates the Pomodoro work/break sessions.
func RunPomodoro() {
	mu.Lock()
	shared.State.Ongoing = true
	shared.State.Paused = false
	mu.Unlock()

	pomodoroConfig := shared.State.Settings

	for session := 1; session <= pomodoroConfig.Sessions; session++ {
		timer = timer.Init()

		shared.State.Session = session

		shared.State.Mode = "Work"
		go timer.Start(pomodoroConfig.Work)
		if !waitForTimer(timer) {
			break
		}

		if session < pomodoroConfig.Sessions {
			shared.State.Mode = "ShortBreak"
			go timer.Start(pomodoroConfig.ShortBreak)
			if !waitForTimer(timer) {
				break
			}
		} else { // last phase, prepare for finish
			shared.State.Mode = "LongBreak"
			go timer.Start(pomodoroConfig.LongBreak)
			if !waitForTimer(timer) {
				break
			}
			shared.State.Mode = "End"
			time.Sleep(time.Second)
		}
	}

	mu.Lock()
	shared.State.Ongoing = false
	shared.State.Paused = false
	mu.Unlock()

	shared.State.Mode = "Idle"
	shared.State.Session = 0
	shared.State.TimeLeft = shared.State.Settings.Work
}

func ResetPomodoro() {
	timer.Stop()
}

func PausePomodoro() {
	mu.Lock()
	shared.State.Paused = true
	mu.Unlock()

	timer.Pause()
}

func ResumePomodoro() {
	mu.Lock()
	shared.State.Paused = false
	mu.Unlock()
	timer.Resume()
}

func IsPomodoroOngoing() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.State.Ongoing
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.State.Paused
}

func UpdateSettings(settings models.PomodoroConfig) {
	if settings != (models.PomodoroConfig{}) {
		shared.State.Settings = settings
		shared.State.TimeLeft = settings.Work
	}
}
